
#include "Common.h"

#define Remake(name) MessageBoxA(NULL, "Your "#name" actions need to be recreated.\r\n" \
									   "This is probably due to parameter changes.", "Lacewing Relay Client - DarkEDIF", MB_OK)
void Extension::Replaced_Connect(char * Hostname, int Port)
{
	Remake("Connect");
}
void Extension::Disconnect()
{
	Client.Disconnect();
}
void Extension::SetName(char * Name)
{
	if (!Name)
		CreateError("Error: SetName() was called with a null parameter.");
	else if (Name[0] == '\0')
		CreateError("Error: SetName() was called with \"\".");
	else
		Client.SetName(Name);
}
void Extension::Replaced_JoinChannel(char * ChannelName, int HideChannel)
{
	Remake("Join channel");
}
void Extension::LeaveChannel()
{
	ThreadData.Channel->Leave();
}
void Extension::SendTextToServer(int Subchannel, char * TextToSend)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Send Text to Server was called with an invalid subchannel; it must be between 0 and 255.");
	else if (!TextToSend)
		CreateError("Error: Send Text to Server was called with a null parameter.");
	else
		Client.SendServer(Subchannel, TextToSend, -1, 0);
}
void Extension::SendTextToChannel(int Subchannel, char * TextToSend)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Send Text to Channel was called with an invalid subchannel; it must be between 0 and 255.");
	else if (!TextToSend)
		CreateError("Error: Send Text to Channel was called with a null parameter.");
	else
		ThreadData.Channel->Send(Subchannel, TextToSend, -1, 0);
}
void Extension::SendTextToPeer(int Subchannel, char * TextToSend)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Send Text to Peer was called with an invalid subchannel; it must be between 0 and 255.");
	else if (!TextToSend)
		CreateError("Error: Send Text to Peer was called with a null parameter.");
	else
		ThreadData.Peer->Send(Subchannel, TextToSend, -1, 0);
}
void Extension::SendNumberToServer(int Subchannel, int NumToSend)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Send Number to Server was called with an invalid subchannel; it must be between 0 and 255.");
	else
		Client.SendServer(Subchannel, (char *)&NumToSend, sizeof(int), 1);
}
void Extension::SendNumberToChannel(int Subchannel, int NumToSend)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Send Number to Channel was called with an invalid subchannel; it must be between 0 and 255.");
	else
		ThreadData.Channel->Send(Subchannel, (char *)&NumToSend, sizeof(int), 1);
}
void Extension::SendNumberToPeer(int Subchannel, int NumToSend)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Send Number to Peer was called with an invalid subchannel; it must be between 0 and 255.");
	else
		ThreadData.Peer->Send(Subchannel, (char *)&NumToSend, sizeof(int), 1);
}
void Extension::BlastTextToServer(int Subchannel, char * TextToSend)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Blast Text to Server was called with an invalid subchannel; it must be between 0 and 255.");
	else if (!TextToSend)
		CreateError("Error: Blast Text to Server was called with a null parameter.");
	else
		ThreadData.Peer->Blast(Subchannel, TextToSend, -1, 0);
}
void Extension::BlastTextToChannel(int Subchannel, char * TextToSend)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Blast Text to Channel was called with an invalid subchannel; it must be between 0 and 255.");
	else if (!TextToSend)
		CreateError("Error: Blast Text to Channel was called with a null parameter.");
	else
		ThreadData.Channel->Blast(Subchannel, TextToSend, -1, 0);
}
void Extension::BlastTextToPeer(int Subchannel, char * TextToSend)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Blast Text to Peer was called with an invalid subchannel; it must be between 0 and 255.");
	else if (!TextToSend)
		CreateError("Error: Blast Text to Peer was called with a null parameter.");
	else
		ThreadData.Peer->Blast(Subchannel, TextToSend, -1, 0);
}
void Extension::BlastNumberToServer(int Subchannel, int NumToSend)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Blast Number to Server was called with an invalid subchannel; it must be between 0 and 255.");
	else
		Client.BlastServer(Subchannel, (char *)&NumToSend, sizeof(int), 1);
}
void Extension::BlastNumberToChannel(int Subchannel, int NumToSend)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Blast Number to Channel was called with an invalid subchannel; it must be between 0 and 255.");
	else
		ThreadData.Channel->Blast(Subchannel, (char *) &NumToSend, 4, 1);
}
void Extension::BlastNumberToPeer(int Subchannel, int NumToSend)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Blast Number to Peer was called with an invalid subchannel; it must be between 0 and 255.");
	else
		ThreadData.Peer->Blast(Subchannel, (char *) &NumToSend, 4, 1);
}
void Extension::SelectChannelWithName(char * ChannelName)
{
	Lacewing::RelayClient::Channel * Selected = Client.FirstChannel();
	while (Selected)
	{
		if (!strcmp(Selected->Name(), ChannelName))
			break;
		Selected = Selected->Next();
	}
	
	// Only modify ThreadData.Channel if we found it
	if (Selected)
		ThreadData.Channel = Selected;
	else
	{
		std::string Error = "Channel not found:\r\n";
		Error += ChannelName;
		CreateError(Error.c_str());
	}
}
void Extension::ReplacedNoParams()
{
	Remake("Unknown action");
}
void Extension::LoopClientChannels()
{
	// Store selected channel
	Lacewing::RelayClient::Channel * Stored = ThreadData.Channel,
								   * Selected = Client.FirstChannel();
	while (Selected)
	{
		SaveExtInfo &S = AddEvent(14);
		S.Channel = Selected;
		Selected = Selected->Next();
	}
	SaveExtInfo &S = AddEvent(18);
	S.Channel = Stored;
}
void Extension::SelectPeerOnChannelByName(char * PeerName)
{
	Lacewing::RelayClient::Channel::Peer * Selected = ThreadData.Channel->FirstPeer();
	while (Selected)
	{
		if (!strcmp(Selected->Name(), PeerName))
			break;
		Selected = Selected->Next();
	}

	// Only modify ThreadData.Peer if we found it
	if (Selected)
		ThreadData.Peer = Selected;
	else
	{
		std::string Error = "Peer not found:\r\n";
		Error += ThreadData.Channel->Name();
		Error += "\r\nOn channel:\r\n";
		Error += PeerName;
		CreateError(Error.c_str());
	}
}
void Extension::SelectPeerOnChannelByID(int PeerID)
{
	if (PeerID < 0)
		CreateError("Could not select peer on channel, ID is below 0.");
	else
	{
		Lacewing::RelayClient::Channel::Peer * Selected = ThreadData.Channel->FirstPeer();
		while (Selected)
		{
			if (Selected->ID() != PeerID)
				break;
			Selected = Selected->Next();
		}

		// Only modify ThreadData.Peer if we found it
		if (Selected)
			ThreadData.Peer = Selected;
		else
		{
			char num[20];
			std::string Error = "Peer ID ";
			if (_itoa_s(PeerID, num, 20, 10))
				Error += "> could not be copied <";
			else
				Error += &num[0];
			Error += " not found on channel:";
			Error += ThreadData.Channel->Name();
			CreateError(Error.c_str());
		}
	}
}
void Extension::LoopPeersOnChannel()
{
	// Store selected channel
	Lacewing::RelayClient::Channel::Peer * Stored = ThreadData.Peer,
										 * Selected = ThreadData.Channel->FirstPeer();
	while (Selected)
	{
		SaveExtInfo &S = AddEvent(13);
		S.Peer = Selected;
		Selected = Selected->Next();
	}
	SaveExtInfo &S = AddEvent(17);
	S.Peer = Stored;
}
void Extension::RequestChannelList()
{
	Client.ListChannels();
}
void Extension::LoopListedChannels()
{
	Lacewing::RelayClient::ChannelListing * Selected = Client.FirstChannelListing();
	while (Selected)
	{
		SaveExtInfo &S = AddEvent(27); // Catch first listing by this being first
		S.ChannelListing = Selected;
		Selected = Selected->Next();
	}
	AddEvent(28);
}
void Extension::SendBinaryToServer(int Subchannel)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Subchannel invalid; it must be between 0 and 255.");
	else
		Client.SendServer(Subchannel, ThreadData.ReceivedMsg.Content, ThreadData.ReceivedMsg.Size, 2);
	
	if (AutomaticallyClearBinary)
		ClearBinaryToSend();
}
void Extension::SendBinaryToChannel(int Subchannel)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Subchannel invalid; it must be between 0 and 255.");
	else
		ThreadData.Channel->Send(Subchannel, SendMsg, SendMsgSize, 2);

	if (AutomaticallyClearBinary)
		ClearBinaryToSend();
}
void Extension::SendBinaryToPeer(int Subchannel)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Subchannel invalid; it must be between 0 and 255.");
	else
		ThreadData.Peer->Send(Subchannel, SendMsg, SendMsgSize, 2);
	
	if (AutomaticallyClearBinary)
		ClearBinaryToSend();
}
void Extension::BlastBinaryToServer(int Subchannel)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Subchannel invalid; it must be between 0 and 255.");
	else
		Client.BlastServer(Subchannel, SendMsg, SendMsgSize, 2);
	
	if (AutomaticallyClearBinary)
		ClearBinaryToSend();
}
void Extension::BlastBinaryToChannel(int Subchannel)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Subchannel invalid; it must be between 0 and 255.");
	else
		ThreadData.Channel->Blast(Subchannel, SendMsg, SendMsgSize, 2);
	
	if (AutomaticallyClearBinary)
		ClearBinaryToSend();
}
void Extension::BlastBinaryToPeer(int Subchannel)
{
	if (Subchannel > 255 || Subchannel < 0)
		CreateError("Error: Subchannel invalid; it must be between 0 and 255.");
	else
		ThreadData.Peer->Blast(Subchannel, SendMsg, SendMsgSize, 2);
	
	if (AutomaticallyClearBinary)
		ClearBinaryToSend();
}
void Extension::AddByteText(char * Byte)
{
	// The inline function will check it fully :)
	AddToSend(Byte, 1);
}
void Extension::AddByteInt(int Byte)
{
	if (Byte > 255 || Byte < -128)
		CreateError("Error: Byte out of scale.");
	else
	{
		if (Byte < 0)
		{
			char RealByte = (char)Byte;
			AddToSend(&RealByte, 1);
		}
		else
		{
			unsigned char RealByte = (unsigned char)Byte;
			AddToSend(&RealByte, 1);
		}
	}
}
void Extension::AddShort(int Short)
{
	if (Short > 65535 || Short < -32768)
		CreateError("Error: Short out of bounds.");
	else
	{
		if (Short < 0)
		{
			short RealShort = (short)Short;
			AddToSend(&RealShort, 2);
		}
		else
		{
			unsigned short RealShort = (unsigned short)Short;
			AddToSend(&RealShort, 2);
		}
	}
}
void Extension::AddInt(int Int)
{
	AddToSend(&Int, 4);
}
void Extension::AddFloat(float Float)
{
	AddToSend(&Float, 4);
}
void Extension::AddStringWithoutNull(char * String)
{
	if (String)
		AddToSend(String, strlen(String));
	else
		CreateError("Adding string failed: pointer was null.");
}
void Extension::AddString(char * String)
{
	AddToSend(String, strlen(String)+1);
}
void Extension::AddBinary(void * Address, int Size)
{
	if (Size <= 0)
		CreateError("Add binary failed: Size <= 0.");
	else
		AddToSend(Address, Size);
}
void Extension::ClearBinaryToSend()
{
	if (SendMsg)
	{
		free(SendMsg);
		SendMsg = NULL;
	}
	SendMsgSize = 0;
}
void Extension::SaveReceivedBinaryToFile(int Position, int Size, char * Filename)
{
	if (Position < 0)
		CreateError("Cannot save received binary; Position less than 0.");
	else if (Size <= 0)
		CreateError("Cannot save received binary; Size equal or less than 0.");
	else if (!Filename || Filename[0] == '\0')
		CreateError("Cannot save received binary; filename is invalid.");
	else if (ThreadData.ReceivedMsg.Size-Size <= 0)
		CreateError("Cannot save received binary; Message is too small.");
	else
	{
		FILE * File = NULL;
		if (fopen_s(&File, Filename, "wb") || !File)
		{
			char errorval [20];
			SaveExtInfo &S = AddEvent(0);
			std::string Error = "Cannot save received binary to file, error ";
			if (_itoa_s(*_errno(), &errorval[0], 20, 10))
			{
				Error += " with opening the file, and with converting error number.";
			}
			else
			{
				Error += "number [";
				Error += &errorval[0];
				Error += "] occured with opening the file.";
			}
			Error += "\r\nThe message has not been modified.";
			S.Error.Text = _strdup(Error.c_str());
			return;
		}
		// Jump to end
		fseek(File, 0, SEEK_END);

		// Read current position as file size
		long long filesize = _ftelli64(File);

		// Go back to start
		fseek(File, 0, SEEK_SET);
		long l;
		if ((l = fwrite(ThreadData.ReceivedMsg.Content, 1, Size, File)) != Size)
		{
			char sizeastext [20];
			SaveExtInfo &S = AddEvent(0);
			std::string Error = "Couldn't save the received binary to file, ";
			if (_itoa_s(*_errno(), &sizeastext[0], 20, 10))
			{
				Error += " and error copying size.";
			}
			else
			{
				Error += &sizeastext[0];
				Error += " bytes managed to be written.";
			}
			S.Error.Text = _strdup(Error.c_str());
		}
		fclose(File);
	}
}
void Extension::AppendReceivedBinaryToFile(int Position, int Size, char * Filename)
{
	if (Position < 0)
		CreateError("Cannot append received binary; Position less than 0.");
	else if (Size <= 0)
		CreateError("Cannot append received binary; Size equal or less than 0.");
	else if (!Filename || Filename[0] == '\0')
		CreateError("Cannot append received binary; filename is invalid.");
	else if (ThreadData.ReceivedMsg.Size-Size <= 0)
		CreateError("Cannot append received binary; Message is too small.");
	else
	{
		// Open while denying write of other programs
		FILE * File = _fsopen(Filename, "ab", SH_DENYWR);
		if (!File)
		{
			char errorval [20];
			SaveExtInfo &S = AddEvent(0);
			std::string Error = "Cannot append received binary to file, error ";
			if (_itoa_s(*_errno(), &errorval[0], 20, 10))
			{
				Error += " with opening the file, and with converting error number.";
			}
			else
			{
				Error += "number [";
				Error += &errorval[0];
				Error += "] occured with opening the file.";
			}
			Error += "\r\nThe message has not been modified.";

			S.Error.Text = _strdup(Error.c_str());
			return;
		}

		long l;
		if ((l = fwrite(ThreadData.ReceivedMsg.Content, 1, Size, File)) != Size)
		{
			char sizeastext [20];
			SaveExtInfo &S = AddEvent(0);
			std::string Error = "Couldn't append the received binary to file, ";
			if (_itoa_s(*_errno(), &sizeastext[0], 20, 10))
			{
				Error += " and error copying size.";
			}
			else
			{
				Error += &sizeastext[0];
				Error += " bytes managed to be append.";
			}
			S.Error.Text = _strdup(Error.c_str());
		}
		fclose(File);
	}
}
void Extension::AddFileToBinary(char * Filename)
{
	if (!Filename || Filename[0] == '\0')
		CreateError("Cannot add file to send binary; filename is invalid.");
	else
	{
		FILE * File = NULL;
		
		// Open and deny other programs write priviledges
		if (!(File = _fsopen(Filename, "wb", _SH_DENYWR)))
		{
			char errorval [20];
			SaveExtInfo &S = AddEvent(0);
			std::string Error = "Cannot save binary to file, error ";
			if (_itoa_s(*_errno(), &errorval[0], 20, 10))
			{
				Error += " with opening the file, and with converting error number.";
			}
			else
			{
				Error += "number [";
				Error += &errorval[0];
				Error += "] occured with opening the file.";
			}
			Error += "\r\nThe message has not been modified.";

			S.Error.Text = _strdup(Error.c_str());
			return;
		}
		// Jump to end
		fseek(File, 0, SEEK_END);

		// Read current position as file size
		long filesize = ftell(File);

		// Go back to start
		fseek(File, 0, SEEK_SET);

		char * buffer = (char *)malloc(filesize);
		if (!buffer)
			CreateError("Couldn't reserve enough memory to add file into message.");
		else
		{
			size_t s;
			if ((s = fread_s(buffer, filesize, 1, filesize, File)) != filesize)
			{
				char sizeastext [20];
				SaveExtInfo &S = AddEvent(0);
				std::string Error = "Couldn't write full buffer to file, ";
				if (_itoa_s(s, &sizeastext[0], 20, 10))
				{
					Error += " and error copying size.";
				}
				else
				{
					Error += &sizeastext[0];
					Error += " bytes managed to be written.";
				}
				S.Error.Text = _strdup(Error.c_str());
			}
			AddToSend(buffer, s);

			free(buffer);
		}
		fclose(File);
	} 
}
void Extension::SelectChannelMaster()
{
	if (!ThreadData.Channel)
		CreateError("Could not select channel master: No channel selected.");
	else
	{
		Lacewing::RelayClient::Channel::Peer * Stored = ThreadData.Peer;
	
		ThreadData.Peer = ThreadData.Channel->FirstPeer();
		while (ThreadData.Peer)
		{
			if (ThreadData.Channel->FirstPeer()->IsChannelMaster())
				return; // Selected the correct peer
			ThreadData.Peer = ThreadData.Peer->Next();
		}

		// Restore if no channel master found
		ThreadData.Peer = Stored;
	}
}
void Extension::JoinChannel(char * ChannelName, int Hidden, int CloseAutomatically)
{
	if (!ChannelName || ChannelName[0] == '\0')
		CreateError("Cannot join channel: invalid channel name supplied.");
	else
		Client.Join(ChannelName, Hidden != 0, CloseAutomatically != 0);
}
void Extension::CompressSendBinary()
{
	// TODO: Zlib compression
}
void Extension::DecompressReceivedBinary()
{
	// 
}
void Extension::MoveReceivedBinaryCursor(int Position)
{
	if (Position < 0)
		CreateError("Cannot move cursor; Position less than 0.");
	else if (ThreadData.ReceivedMsg.Size - Position <= 0)
		CreateError("Cannot move cursor; Message is too small.");
	else
		ThreadData.ReceivedMsg.Cursor = Position;
}
void Extension::LoopListedChannelsWithLoopName(char * LoopName)
{
	if (!LoopName || LoopName[0] == '\0')
		CreateError("Cannot loop listed channels: invalid loop name supplied.");
	else
	{
		Lacewing::RelayClient::ChannelListing * Selected = Client.FirstChannelListing();
		while (Selected)
		{
			SaveExtInfo &S = AddEvent(59); // Catch first listing by this being first
			S.ChannelListing = Selected;
			S.Loop.Name = _strdup(LoopName);
			Selected = Selected->Next();
		}
	}
	SaveExtInfo &S = AddEvent(60);
	S.Loop.Name = _strdup(LoopName);
}
void Extension::LoopClientChannelsWithLoopName(char * LoopName)
{
	Lacewing::RelayClient::Channel * Stored = ThreadData.Channel,
								   * LoopChannel = Client.FirstChannel();
	while (LoopChannel)
	{
		SaveExtInfo &S = AddEvent(63);
		S.Channel = LoopChannel;
		S.Loop.Name = _strdup(LoopName);
		LoopChannel = LoopChannel->Next();
	}
	SaveExtInfo &S = AddEvent(64);
	S.Channel = Stored;
	S.Loop.Name = _strdup(LoopName);
}
void Extension::LoopPeersOnChannelWithLoopName(char * LoopName)
{
	Lacewing::RelayClient::Channel::Peer * Stored = ThreadData.Peer,
										 * LoopPeer = ThreadData.Channel->FirstPeer();
	while (LoopPeer)
	{
		SaveExtInfo &S = AddEvent(61);
		S.Peer = LoopPeer;
		S.Loop.Name = _strdup(LoopName);
	}
	SaveExtInfo &S = AddEvent(62);
	S.Peer = Stored;
	S.Loop.Name = _strdup(LoopName);
}
void Extension::Connect(char * Hostname)
{
	if (!Hostname || Hostname[0] == '\0')
		CreateError("Cannot connect to server: invalid hostname supplied.");
	int Port = 6121;
	if (strchr(Hostname, ':'))
	{
		Port = atoi(strchr(Hostname, ':')+1);
		
		if (Port < 0 || Port > 35565)
			return CreateError("Invalid port in hostname: too many numbers. Ports are limited from 0 to 35565.");
	}
	Client.Connect(Hostname, Port);
}
void Extension::ResizeBinaryToSend(int NewSize)
{
	if (NewSize < 0)
		CreateError("Cannot change size of binary to send: new size is under 0 bytes.");
	else
	{
		char * NewMsg = (char *)realloc(SendMsg, NewSize);
		if (!NewMsg)
		{
			return CreateError("Cannot change size of binary to send: reallocation of memory failed.\r\n"
							   "Size has not been modified.");
		}
		// Clear new bytes to 0
		memset(NewMsg + SendMsgSize, 0, NewSize - SendMsgSize);
			
		SendMsg = NewMsg;
		SendMsgSize = NewSize;
	}
}
