
#include "Common.h"

const char * Extension::Error()
{
	return Runtime.CopyString(ThreadData.Error.Text);
}

const char * Extension::ReplacedExprNoParams()
{
	return Runtime.CopyString("");
}

const char * Extension::Self_Name()
{
	return Runtime.CopyString(Client.Name() ? Client.Name() : "");
}

int Extension::Self_ChannelCount()
{
	return Client.ChannelCount();
}

const char * Extension::Peer_Name()
{
	return Runtime.CopyString((ThreadData.Peer && ThreadData.Peer->Name()) ? ThreadData.Peer->Name() : "");
}

const char * Extension::ReceivedStr()
{
	if (ThreadData.ReceivedMsg.Content[ThreadData.ReceivedMsg.Size-1] != '\0') 
	{
		CreateError("Received$() was used on a message that is not null-terminated.");
		return Runtime.CopyString("");
	}
	else
		return Runtime.CopyString(ThreadData.ReceivedMsg.Content);
}

int Extension::ReceivedInt()
{
	if (ThreadData.ReceivedMsg.Size != 4) 
	{
		CreateError("Received() was used on a message that is not a number message.");
		return 0;
	}
	else
		return *(int *)ThreadData.ReceivedMsg.Content;
}

int Extension::Subchannel()
{
	return (int)ThreadData.ReceivedMsg.Subchannel;
}

int Extension::Peer_ID()
{
	return ThreadData.Peer ? ThreadData.Peer->ID() : -1;
}

const char * Extension::Channel_Name()
{
	return Runtime.CopyString(ThreadData.Channel ? ThreadData.Channel->Name() : ThreadData.Loop.Name ? ThreadData.Loop.Name : "");
}

int Extension::Channel_PeerCount()
{
	return ThreadData.Channel ? ThreadData.Channel->PeerCount() : -1;
}

const char * Extension::ChannelListing_Name()
{
	return Runtime.CopyString(ThreadData.ChannelListing ? ThreadData.Channel->Name() : "");
}

int Extension::ChannelListing_PeerCount()
{
	return ThreadData.Channel ? ThreadData.Channel->PeerCount() : -1;
}

int Extension::Self_ID()
{
	return Client.ID();
}

const char * Extension::StrByte(int Index)
{
	if (Index < 0)
	{
		CreateError("Could not read from received binary, index less than 0.");
		return Runtime.CopyString("");
	}
	else if (ThreadData.ReceivedMsg.Size - Index < sizeof(char))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return Runtime.CopyString("");
	}
	else
		return Runtime.CopyString(std::string(ThreadData.ReceivedMsg.Content+Index, 1).c_str());
}

unsigned int Extension::UnsignedByte(int Index)
{
	if (Index < 0)
	{
		CreateError("Could not read from received binary, index less than 0.");
		return 0;
	}
	else if (ThreadData.ReceivedMsg.Size - Index < sizeof(unsigned char))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return 0;
	}
	else
		return (unsigned int)(*(unsigned char *)(ThreadData.ReceivedMsg.Content+Index));
}

int Extension::SignedByte(int Index)
{
	if (Index < 0)
	{
		CreateError("Could not read from received binary, index less than 0.");
		return 0;
	}
	else if (ThreadData.ReceivedMsg.Size - Index < sizeof(char))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return 0;
	}
	else
		return (int)(ThreadData.ReceivedMsg.Content+Index);
}

unsigned int Extension::UnsignedShort(int Index)
{
	if (Index < 0)
	{
		CreateError("Could not read from received binary, index less than 0.");
		return 0;
	}
	else if (ThreadData.ReceivedMsg.Size - Index < sizeof(unsigned short))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return 0;
	}
	else
		return (unsigned int)(*(unsigned short *)(ThreadData.ReceivedMsg.Content + Index));
}

int Extension::SignedShort(int Index)
{
	if (Index < 0)
	{
		CreateError("Could not read from received binary, index less than 0.");
		return 0;
	}
	else if (ThreadData.ReceivedMsg.Size - Index < sizeof(short))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return 0;
	}
	else
		return (int)(*(short *)(ThreadData.ReceivedMsg.Content + Index));
}

unsigned int Extension::UnsignedInteger(int Index)
{
	if (Index < 0)
	{
		CreateError("Could not read from received binary, index less than 0.");
		return 0;
	}
	else if (ThreadData.ReceivedMsg.Size - Index < sizeof(unsigned int))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return 0;
	}
	else
		return (*(unsigned int *)(ThreadData.ReceivedMsg.Content + Index));
}

int Extension::SignedInteger(int Index)
{
	if (Index < 0)
	{
		CreateError("Could not read from received binary, index less than 0.");
		return 0;
	}
	else if (ThreadData.ReceivedMsg.Size - Index < sizeof(int))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return 0;
	}
	else
		return (*(int *)(ThreadData.ReceivedMsg.Content + Index));
}

float Extension::Float(int Index)
{
	if (Index < 0)
	{
		CreateError("Could not read from received binary, index less than 0.");
		return 0.0f;
	}
	else if (ThreadData.ReceivedMsg.Size - Index < sizeof(float))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return 0.0f;
	}
	else
		return (*(float *)(ThreadData.ReceivedMsg.Content + Index));
}

const char * Extension::StringWithSize(int Index, int Size)
{
	if (Index < 0)
	{
		CreateError("Could not read from received binary, index less than 0.");
		return Runtime.CopyString("");
	}
	else if (ThreadData.ReceivedMsg.Size - Index < Size * sizeof(TCHAR))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return Runtime.CopyString("");
	}
	else
		return Runtime.CopyString(std::string(ThreadData.ReceivedMsg.Content+Index, Size).c_str());
}

const char * Extension::String(int Index)
{
	if (Index < 0)
	{
		CreateError("Could not read from received binary, index less than 0.");
		return Runtime.CopyString("");
	}
	else if (ThreadData.ReceivedMsg.Size - Index < 1)
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return Runtime.CopyString("");
	}
	else if (strnlen(ThreadData.ReceivedMsg.Content + Index, ThreadData.ReceivedMsg.Size - Index + 1) == ThreadData.ReceivedMsg.Size - Index + 1)
	{
		CreateError("Could not read null-terminated string; null terminator not found.");
		return Runtime.CopyString("");
	}
	else
		return Runtime.CopyString(ThreadData.ReceivedMsg.Content+Index);
}

int Extension::ReceivedBinarySize()
{
	return ThreadData.ReceivedMsg.Size;
}

const char * Extension::Lacewing_Version()
{
	return Runtime.CopyString(Lacewing::Version());
}

int Extension::SendBinarySize()
{
	return SendMsgSize;
}

const char * Extension::Self_PreviousName()
{
	return Client.Tag ? (char *)Client.Tag : "";
}

const char * Extension::Peer_PreviousName()
{
	return ThreadData.Peer->Tag ? (char *)ThreadData.Peer->Tag : "";
}

const char * Extension::DenyReason()
{
	return Runtime.CopyString(DenyReasonBuffer ? DenyReasonBuffer : "No reason specified.");
}

const char * Extension::HostIP()
{
	return Client.ServerAddress().ToString();
}

int Extension::HostPort()
{
	return Client.ServerAddress().Port();
}

const char * Extension::WelcomeMessage()
{
	return Client.WelcomeMessage(); 
}

long Extension::ReceivedBinaryAddress()
{
	return (long)ThreadData.ReceivedMsg.Content;
}

const char * Extension::CursorStrByte()
{
	if (ThreadData.ReceivedMsg.Size - ThreadData.ReceivedMsg.Cursor < sizeof(char))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return Runtime.CopyString("");
	}
	else
		return Runtime.CopyString(std::string(ThreadData.ReceivedMsg.Content+ThreadData.ReceivedMsg.Cursor, 1).c_str());
}

unsigned int Extension::CursorUnsignedByte()
{
	if (ThreadData.ReceivedMsg.Size - ThreadData.ReceivedMsg.Cursor < sizeof(unsigned char))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read.");
		return 0;
	}
	else
	{
		++ThreadData.ReceivedMsg.Cursor;
		return (unsigned int)(*(unsigned char *)(ThreadData.ReceivedMsg.Content + (ThreadData.ReceivedMsg.Cursor-1)));
	}
}

int Extension::CursorSignedByte()
{
	if (ThreadData.ReceivedMsg.Size - ThreadData.ReceivedMsg.Cursor < sizeof(char))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read.");
		return 0;
	}
	else
	{
		++ThreadData.ReceivedMsg.Cursor;
		return (int)(ThreadData.ReceivedMsg.Content+ThreadData.ReceivedMsg.Cursor-1);
	}
}

unsigned int Extension::CursorUnsignedShort()
{
	if (ThreadData.ReceivedMsg.Size - ThreadData.ReceivedMsg.Cursor < sizeof(unsigned short))
	{
		CreateError("Could not read from received binary, index less than 0.");
		return 0;
	}
	else
	{
		ThreadData.ReceivedMsg.Cursor += 2;
		return (unsigned int)(*(unsigned short *)(ThreadData.ReceivedMsg.Content + ThreadData.ReceivedMsg.Size - 2));
	}
}

int Extension::CursorSignedShort()
{
	if (ThreadData.ReceivedMsg.Size - ThreadData.ReceivedMsg.Cursor < sizeof(short))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return 0;
	}
	else
	{
		ThreadData.ReceivedMsg.Cursor += 2;
		return (int)(*(short *)(ThreadData.ReceivedMsg.Content + ThreadData.ReceivedMsg.Cursor - 2));
	}
}

unsigned int Extension::CursorUnsignedInteger()
{
	if (ThreadData.ReceivedMsg.Size - ThreadData.ReceivedMsg.Cursor < sizeof(unsigned int))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return 0;
	}
	else
	{
		ThreadData.ReceivedMsg.Cursor += 4;
		return (*(unsigned int *)(ThreadData.ReceivedMsg.Content + ThreadData.ReceivedMsg.Cursor - 4));
	}
}

int Extension::CursorSignedInteger()
{
	if (ThreadData.ReceivedMsg.Size - ThreadData.ReceivedMsg.Cursor < sizeof(int))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return 0;
	}
	else
	{
		ThreadData.ReceivedMsg.Cursor += 4;
		return (*(int *)(ThreadData.ReceivedMsg.Content + ThreadData.ReceivedMsg.Cursor - 4));
	}
}

float Extension::CursorFloat()
{
	if (ThreadData.ReceivedMsg.Size - ThreadData.ReceivedMsg.Cursor < sizeof(float))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return 0.0f;
	}
	else
	{
		ThreadData.ReceivedMsg.Cursor += 4;
		return (*(float *)(ThreadData.ReceivedMsg.Content + ThreadData.ReceivedMsg.Cursor - 4));
	}
}

const char * Extension::CursorStringWithSize(int Size)
{
	if (ThreadData.ReceivedMsg.Size - ThreadData.ReceivedMsg.Cursor < Size * sizeof(TCHAR))
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return Runtime.CopyString("");
	}
	else
	{
		ThreadData.ReceivedMsg.Cursor += Size;
		return Runtime.CopyString(std::string(ThreadData.ReceivedMsg.Content + ThreadData.ReceivedMsg.Cursor - Size, Size).c_str());
	}
}

const char * Extension::CursorString()
{
	if (ThreadData.ReceivedMsg.Size - ThreadData.ReceivedMsg.Cursor < 1)
	{
		CreateError("Could not read from received binary, message is smaller than variable to be read. Check your index.");
		return Runtime.CopyString("");
	}
	else if (strnlen(ThreadData.ReceivedMsg.Content + ThreadData.ReceivedMsg.Cursor, ThreadData.ReceivedMsg.Size - ThreadData.ReceivedMsg.Cursor + 1) == ThreadData.ReceivedMsg.Size - ThreadData.ReceivedMsg.Cursor + 1)
	{
		CreateError("Could not read null-terminated string; null terminator not found.");
		return Runtime.CopyString("");
	}
	else
	{
		size_t s = ThreadData.ReceivedMsg.Cursor;
		ThreadData.ReceivedMsg.Cursor += strlen(ThreadData.ReceivedMsg.Content + ThreadData.ReceivedMsg.Cursor)+1;
		return Runtime.CopyString(ThreadData.ReceivedMsg.Content+s);
	}
}

long Extension::SendBinaryAddress()
{
	return (long)SendMsg;
}
