
//////////////////////////////////////////////////////////////////////////////
//
// Macros
//

#ifndef _BlitMac_h
#define	_BlitMac_h

#ifndef __GNUC__
#pragma pack(push,1)
#endif // __GNUC__

//////////////////////////////////////////
//
// MAC
//
#ifdef _MAC
// 24 bit pixel in memory screen
typedef struct RGBSCREEN24 {
	BYTE	red;
	BYTE	green;
	BYTE	blue;
} RGBSCREEN24;
typedef RGBSCREEN24 *LPRGBSCREEN24;

// 32 bit pixel in memory screen
typedef struct RGBSCREEN32 {
	BYTE	unused;
	BYTE	red;
	BYTE	green;
	BYTE	blue;
} RGBSCREEN32;
typedef RGBSCREEN32 *LPRGBSCREEN32;

//////////////////////////////////////////
//
// PC
//
#else
// 24 bit pixel in memory screen
typedef struct RGBSCREEN24 {
	BYTE	blue;
	BYTE	green;
	BYTE	red;
#ifdef __GNUC__
} __attribute__((packed,aligned(1))) RGBSCREEN24;
#else
} RGBSCREEN24;
#endif // __GNUC__
typedef RGBSCREEN24 *LPRGBSCREEN24;

// 32 bit pixel in memory screen
typedef struct RGBSCREEN32 {
	BYTE	blue;
	BYTE	green;
	BYTE	red;
	BYTE	unused;
#ifdef __GNUC__
} __attribute__((packed,aligned(1))) RGBSCREEN32;
#else
} RGBSCREEN32;
#endif // __GNUC__
typedef RGBSCREEN32 *LPRGBSCREEN32;

#endif // _MAC

// Define (only) one of the following cstes dependending on the location of the unused byt in 32 bit true color mode
//#define	PIXEL32BIT_FIRSTBYTEUNUSED
#define	PIXEL32BIT_LASTBYTEUNUSED

#ifndef __GNUC__
#pragma pack(pop)
#endif

// MACROS

#ifdef _MAC
#define	PALETTEENTRY_TO_DWORD(pte) ((((unsigned int)((LPPALETTEENTRY)pte)->peRed)<<16)|(((unsigned int)((LPPALETTEENTRY)pte)->peGreen)<<8)|(((unsigned int)((LPPALETTEENTRY)pte)->peBlue)))
#define	PALETTEENTRY_TO_COLORREF(pte) ((((unsigned int)((LPPALETTEENTRY)pte)->peRed)<<16)|(((unsigned int)((LPPALETTEENTRY)pte)->peGreen)<<8)|(((unsigned int)((LPPALETTEENTRY)pte)->peBlue)))
#else
// Transforms a PALETTEENTRY to unsigned int, removing the PALETTEENTRY flags
// Note: the placement if the colors in the unsigned int is not important, it's not a COLORREF.
#define	PALETTEENTRY_TO_DWORD(pte) (*((LPDWORD)(pte)) & 0xFFFFFF)

// Transforms a PALETTEENTRY to COLORREF, removing the PALETTEENTRY flags
#define	PALETTEENTRY_TO_COLORREF(pte) (*((LPDWORD)(pte)) & 0xFFFFFF)
#endif // _MAC

// Transforms COLORREF to RGB 15 bit
#define	Rgb24To15(c) (UINT)((((c) & 0xF8) << 7) | (((c) & 0xF800) >> 6) | (((c) & 0xF80000) >> 19))

// Transforms COLORREF to RGB 16 bit
#define	Rgb24To16(c) (UINT)((((c) & 0xF8) << 8) | (((c) & 0xFC00) >> 5) | (((c) & 0xF80000) >> 19))

// Transforms RGB 15 bit to COLORREF
#define	Rgb15To24(w) (UINT)(((unsigned int)((w) & 0x7C00) >> 7) | ((unsigned int)((w) & 0x03E0) << 6) | ((unsigned int)((w) & 0x001F) << 19))

// Transforms RGB 16 bit to COLORREF
#define	Rgb16To24(w) (UINT)(((unsigned int)((w) & 0xF800) >> 8) | ((unsigned int)((w) & 0x07E0) << 5) | ((unsigned int)((w) & 0x001F) << 19))

// Mask to convert a PALETTEENTRY to a unsigned int
#define COLORMASK_PALETTEENTRY_TO_DWORD 0x00FFFFFF

// Transforms PALETTEENTRY to RGB 15 bit
#define	PALETTEENTRY_TO_RGB15(p) (UINT)(((unsigned int)((p)->peRed & 0xF8) << 7) | ((unsigned int)((p)->peGreen & 0xF8) << 2) | ((unsigned int)((p)->peBlue & 0xF8) >> 3))

// Transforms PALETTEENTRY to RGB 16 bit
#define	PALETTEENTRY_TO_RGB16(p) (UINT)(((unsigned int)((p)->peRed & 0xF8) << 8) | ((unsigned int)((p)->peGreen & 0xFC) << 3) | ((unsigned int)((p)->peBlue & 0xF8) >> 3))

// Transforms RGBSCREEN24 to RGB 15 bit
#define	RGBSCREEN24_TO_RGB15(p) (UINT)(((unsigned int)((p)->red & 0xF8) << 7) | ((unsigned int)((p)->green & 0xF8) << 2) | ((unsigned int)((p)->blue & 0xF8) >> 3))

// Transforms RGBSCREEN24 to RGB 16 bit
#define	RGBSCREEN24_TO_RGB16(p) (UINT)(((unsigned int)((p)->red & 0xF8) << 8) | ((unsigned int)((p)->green & 0xFC) << 3) | ((unsigned int)((p)->blue & 0xF8) >> 3))

// Transforms RGBSCREEN24 to COLORREF
#define	RGBSCREEN24_TO_COLORREF(p) (UINT)(((unsigned int)(p)->blue << 16) | ((unsigned int)(p)->green << 8) | ((unsigned int)(p)->red))

// Transforms RGBSCREEN24 to COLORREF
#define	RGBSCREEN32_TO_COLORREF(p) (UINT)(((unsigned int)(p)->blue << 16) | ((unsigned int)(p)->green << 8) | ((unsigned int)(p)->red))

// Transforms RGBSCREEN32 to RGB 15 bit
#define	RGBSCREEN32_TO_RGB15(p) RGBSCREEN24_TO_RGB15(p)

// Transforms RGBSCREEN32 to RGB 16 bit
#define	RGBSCREEN32_TO_RGB16(p) RGBSCREEN24_TO_RGB16(p)

// Transforms COLORREF to unsigned int containing an RGBSCREEN32 for instructions of type "*pdw++ = dw"
#define COLORREF_TO_DWRGBSCREEN32(c) ((unsigned int)GetBValue(c) + ((unsigned int)GetGValue(c) << 8) + ((unsigned int)GetRValue(c) << 16))
#define COLORMASK_DWRGBSCREEN32 0x00FFFFFF

#endif	// _BlitMac_h
