
#ifndef _BFile_h
#define	_BFile_h

#ifndef LPLPVOID
typedef LPVOID *LPLPVOID;
#endif

#include "CfcError.h"
#include "StdDefs.h"

#ifndef OF_READ
#define OF_READ             0x00000000
#define OF_WRITE            0x00000001
#define OF_READWRITE        0x00000002
#define OF_SHARE_COMPAT     0x00000000
#define OF_SHARE_EXCLUSIVE  0x00000010
#define OF_SHARE_DENY_WRITE 0x00000020
#define OF_SHARE_DENY_READ  0x00000030
#define OF_SHARE_DENY_NONE  0x00000040
#endif

STDDLL_API HFILE WINAPI	File_OpenA(const char * fname, int mode);
STDDLL_API HFILE WINAPI	File_CreateA(const char * fname);
STDDLL_API int WINAPI	File_Read (HFILE hf, LPVOID buf, unsigned int len);
STDDLL_API int WINAPI	File_ReadAndCount (HFILE hf, LPVOID buf, unsigned int len);
STDDLL_API int WINAPI	File_ReadShortIntelData (HFILE hf, LPLPVOID pBuf);
STDDLL_API int WINAPI	File_ReadShortIntelString (HFILE hf, LPLPVOID pBuf);
STDDLL_API int WINAPI	File_ReadLongIntelData (HFILE hf, LPLPVOID pBuf);
STDDLL_API int WINAPI	File_Write (HFILE hf, LPVOID buf, unsigned int len);
STDDLL_API unsigned int WINAPI	File_GetPosition (HFILE hf);
STDDLL_API unsigned int WINAPI	File_SeekBegin (HFILE hf, long pos);
STDDLL_API unsigned int WINAPI	File_SeekCurrent (HFILE hf, long pos);
STDDLL_API unsigned int WINAPI	File_SeekEnd (HFILE hf, long pos);
STDDLL_API long WINAPI	File_GetLength ( HFILE hf );
STDDLL_API void WINAPI	File_Close(HFILE hf);
STDDLL_API BOOL WINAPI	File_ExistA(const char * pName);

STDDLL_API HFILE WINAPI	File_OpenW(const wchar_t * fname, int mode);
STDDLL_API HFILE WINAPI	File_CreateW(const wchar_t * fname);
STDDLL_API BOOL WINAPI	File_ExistW(const wchar_t * pName);

#ifdef _UNICODE
#define File_Open File_OpenW
#define File_Create File_CreateW
#define File_Exist File_ExistW
#else
#define File_Open File_OpenA
#define File_Create File_CreateA
#define File_Exist File_ExistA
#endif

#ifndef _MAC
#define	File_ReadIntelWord(h,p,l)	File_Read(h,p,l)
#define	File_ReadIntelDWord(h,p,l)	File_Read(h,p,l)
#else
int		File_ReadIntelWord(HFILE hf, LPVOID buf, unsigned int len);
int		File_ReadIntelDWord(HFILE hf, LPVOID buf, unsigned int len);
#endif

// Input file - abstract class
class STDDLL_API CInputFile
{
	public:
		virtual			~CInputFile() {};

		virtual void	Delete();

		virtual	int		Read(LPBYTE dest, unsigned long lsize) = 0;
		virtual	int		Read(LPBYTE dest, unsigned long lsize, LPDWORD pRead) = 0;
		virtual	int		ReadByte(LPBYTE dest) = 0;
		virtual	int		ReadIntelWord(LPWORD dest);
		virtual	int		ReadIntelDWord(LPDWORD dest);
		virtual	int		ReadMacWord(LPWORD dest);
		virtual	int		ReadMacDWord(LPDWORD dest);
		virtual	int		ReadIntelFloat(PFLOAT dest);
		virtual	int		ReadMacFloat(PFLOAT dest);
		virtual	int		ReadIntelWordArray(LPWORD dest, int count);
		virtual	int		ReadIntelDWordArray(LPDWORD dest, int count);
		virtual	int		ReadIntelFloatArray(PFLOAT dest, int count);
		virtual	int		ReadMacWordArray(LPWORD dest, int count);
		virtual	int		ReadMacDWordArray(LPDWORD dest, int count);
		virtual	int		ReadMacFloatArray(PFLOAT dest, int count);

		virtual long	GetPosition() = 0;
		virtual long	GetLength() = 0;
		virtual	long	Seek(long pos, int method) = 0;

		virtual	int		Attach(HANDLE hnd) = 0;
		virtual	HANDLE	Detach() = 0;

		virtual LPBYTE	GetBuffer(UINT nSize);
		virtual void	FreeBuffer(LPBYTE buf);

		virtual	char *	GetFileNameA() = 0;
		virtual	wchar_t *	GetFileNameW() = 0;

		#ifdef _CFCFILE_UNICODE_DEFS
			#if defined(_UNICODE)
				#define GetFileName GetFileNameW
			#else
				#define GetFileName GetFileNameA
			#endif
		#endif
};
typedef CInputFile * LPINPUTFILE;

// Bufferized input file
class STDDLL_API CInputBufFile : public CInputFile
{
	public:
						CInputBufFile();
		virtual			~CInputBufFile();

		static CInputBufFile*	NewInstance();

				int		Create(HFILE hf);
				int		Create(HFILE hf, unsigned int dwOffset, unsigned int dwSize);
				int		Create(const char * filename);
				int		Create(const char * filename, unsigned int dwOffset, unsigned int dwSize);
				int		Create(const wchar_t * filename);
				int		Create(const wchar_t * filename, unsigned int dwOffset, unsigned int dwSize);

		virtual	int		Read(LPBYTE dest, unsigned int lsize);
		virtual	int		Read (LPBYTE dest, unsigned int lsize, LPDWORD pRead);
		virtual	int		ReadByte(LPBYTE dest);
//		virtual	int		ReadWord(LPWORD dest);
//		virtual	int		ReadDWord(LPDWORD dest);
		virtual long	GetPosition();
		virtual long	GetLength();
		virtual	long	Seek(long pos, int method);
	
		virtual	int		Attach(HANDLE hnd);
		virtual	HANDLE	Detach();

//		virtual LPBYTE	GetBuffer(UINT nSize);
//		virtual void	FreeBuffer(LPBYTE buf);

		virtual	char *	GetFileNameA();
		virtual	wchar_t *	GetFileNameW();

	protected:
				int		Attach(HANDLE hnd, unsigned int dwOffset, unsigned int dwSize);
	private:
		HFILE	m_hf;
		unsigned int	m_curpos;
		LPBYTE	m_buffer;
		LPBYTE	m_bufcurr;
		unsigned int	m_remains;
		wchar_t *	m_fnameW;
		char *	m_fnameA;

		unsigned int	m_startOffset;
		unsigned int	m_length;
};
typedef	CInputBufFile * LPINPUTBUFFILE;

#define	BUFFILE_BUFFER_SIZE	16384

// Memory input file
class STDDLL_API CInputMemFile : public CInputFile
{
	public:
						CInputMemFile();
		virtual			~CInputMemFile();

		static CInputMemFile*	NewInstance();

		int				Create(LPBYTE buffer, unsigned long lsize);
		int				Create(unsigned long lsize);
		LPBYTE			GetMemBuffer();

		virtual	int		Read(LPBYTE dest, unsigned long lsize);
		virtual	int		Read (LPBYTE dest, unsigned long lsize, LPDWORD pRead);
		virtual	int		ReadByte(LPBYTE dest);
//		virtual	int		ReadWord(LPWORD dest);
//		virtual	int		ReadDWord(LPDWORD dest);
		virtual long	GetPosition();
		virtual long	GetLength();
		virtual	long	Seek(long pos, int method);
	
		virtual	int		Attach(HANDLE hnd);
		virtual	HANDLE	Detach();

		virtual LPBYTE	GetBuffer(UINT nSize);
		virtual void	FreeBuffer(LPBYTE buf);

		virtual	char *	GetFileNameA() { return NULL; }
		virtual	wchar_t *	GetFileNameW() { return NULL; }

	private:
		LPBYTE	m_buffer;
		LPBYTE	m_bufcurr;
		unsigned int	m_curpos;
		unsigned int	m_remains;
		BOOL	m_bAutoDelete;
};
typedef	CInputMemFile * LPINPUTMEMFILE;

// Output file: base class
class STDDLL_API COutputFile
{
	public:
		COutputFile() {};
		virtual ~COutputFile() {};

//		virtual int		WriteByte(BYTE b) = 0;
//		virtual int		WriteWord(WORD b) = 0;

		int				WriteIntelWord(LPWORD pw);
		int				WriteIntelDWord(LPDWORD pdw);
		int				WriteIntelFloat(PFLOAT dest);
		int				WriteMacWord(LPWORD pw);
		int				WriteMacDWord(LPDWORD pdw);

		virtual int		Write(LPBYTE pb, UINT sz) = 0;
		virtual int		Flush() = 0;
		virtual unsigned int	GetLength() = 0;

		virtual long	GetPosition() = 0;
		virtual long	Seek(long pos, int method) = 0;

		virtual	char *	GetFileNameA() = 0;
		virtual	wchar_t *	GetFileNameW() = 0;
};
typedef COutputFile * LPOUTPUTFILE;

// Memory output file
class STDDLL_API COutputMemFile : public COutputFile
{
	public:
		COutputMemFile();
		virtual ~COutputMemFile();

		int		Create(UINT nBlockSize = 512);
		int		Create(LPBYTE buffer, unsigned int nBufferSize = 0x7FFFFFF);
		LPBYTE	GetBuffer();
		LPBYTE	DetachBuffer();
		static void FreeBuffer(LPBYTE pBuffer);	// car il faut librer  l'interieur de la DLL

		virtual int		Write(LPBYTE pb, UINT sz);
		virtual int		Flush();
		virtual unsigned int	GetLength();

		virtual long	GetPosition();
		virtual long	Seek(long pos, int method);

		virtual	char *	GetFileNameA() { return NULL; }
		virtual	wchar_t *	GetFileNameW() { return NULL; }

	private:
		LPBYTE	m_buffer;
		LPBYTE	m_curptr;
		unsigned int	m_totalsize;
		unsigned int	m_cursize;
		unsigned int	m_blocksize;
		BOOL	m_bReallocable;
};

// Bufferized output file
class STDDLL_API COutputBufFile : public COutputFile
{
	public:
		COutputBufFile();
		virtual ~COutputBufFile();

		int Create(HFILE hf, UINT nBufferSize = 4096);
		int Create(const char * fname, UINT nBufferSize = 4096);
		int Create(const wchar_t * fname, UINT nBufferSize = 4096);

		virtual int		Write(LPBYTE pb, UINT sz);
		virtual int		Flush();
		virtual unsigned int	GetLength();

		virtual long	GetPosition();
		virtual long	Seek(long pos, int method);

		virtual	char *	GetFileNameA();
		virtual	wchar_t *	GetFileNameW();

	private:
		HFILE	m_hf;
		wchar_t *	m_fnameW;
		LPBYTE	m_buffer;
		LPBYTE	m_curptr;
		unsigned int	m_cursize;
		unsigned int	m_buffersize;
		BOOL	m_bBuffered;
		char *	m_fnameA;
};

#endif	// _BFile_h

