
#ifndef _FilterMgr_h
#define _FilterMgr_h

#include "StdDefs.h"
#if !defined(STDDLL_IMPORTS)
	#include "FilterInfo.h"
#else
	typedef	LPVOID LPFILTERINFOARRAY;
#endif

// Forwards
class CInputFile;

///////////////////////////////////////////////////
//
// Filter Manager base class
//

#define FILTERID_PNG	DWORDSTR('P','N','G',' ')

// Filter Manager
class STDDLL_API CFilterMgr
{
// Public
public:
	CFilterMgr ();
	virtual ~CFilterMgr();

	static CFilterMgr* CreateInstance();

	// Initialization
	virtual void		Initialize(const char * pFilterPath, const char * pFileType, unsigned int dwFlags);
	virtual void		Free();
	virtual void		Delete();

	// Filter info
	virtual int			GetFilterCount();
	virtual const char *		GetFilterNameA(int nIndex);
	virtual const char *		GetFilterFileNameA(int nIndex);
	virtual unsigned int		GetFilterID(int nIndex);
	virtual int			GetFilterIndex(unsigned int dwID);
	virtual const char **		GetFilterExtsA(int nIndex);
	virtual const char **		GetFilterDependenciesA(int nIndex);
	virtual BOOL		CanReadFile(int nIndex, CInputFile* pif);

	// Filter object creation
	virtual LPVOID		CreateFilterObject(int nIndex);
	virtual LPVOID		CreateFilterObject(unsigned int dwID);
	virtual LPVOID		CreateFilterObject(CInputFile* pif);

	// Update
	virtual void		RemoveFilter(int nIndex);

	// Unicode
	virtual void		Initialize(const wchar_t * pFilterPath, const wchar_t * pFileType, unsigned int dwFlags);
	virtual const wchar_t *		GetFilterNameW(int nIndex);
	virtual const wchar_t *		GetFilterFileNameW(int nIndex);
	virtual const wchar_t **	GetFilterExtsW(int nIndex);
	virtual const wchar_t **	GetFilterDependenciesW(int nIndex);

	#ifdef _CFCFILTERMGR_UNICODE_DEFS
		#ifdef _UNICODE
			#define GetFilterName GetFilterNameW
			#define GetFilterFileName GetFilterFileNameW
			#define GetFilterExts GetFilterExtsW
			#define GetFilterDependencies GetFilterDependenciesW
		#else
			#define GetFilterName GetFilterNameA
			#define GetFilterFileName GetFilterFileNameA
			#define GetFilterExts GetFilterExtsA
			#define GetFilterDependencies GetFilterDependenciesA
		#endif
	#endif

	// Data
public:
	LPFILTERINFOARRAY	m_pFilterArray;
};

#endif // _FilterMgr_h
